package com.vendor.burn.serial.workers;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.vendor.burn.Messages;
import com.vendor.burn.analysis.TimeLogger;
import com.vendor.burn.commands.CrcCommand;
import com.vendor.burn.commands.EraseCommand;
import com.vendor.burn.commands.I2cCommand;
import com.vendor.burn.commands.INFOCommand;
import com.vendor.burn.commands.MWCommand;
import com.vendor.burn.commands.MmcPartCommand;
import com.vendor.burn.commands.NetCommand;
import com.vendor.burn.commands.SFCommand;
import com.vendor.burn.commands.SerialCommand;
import com.vendor.burn.commands.TFTPCommand;
import com.vendor.burn.commands.UnZipCommand;
import com.vendor.burn.commands.WriteCommand;
import com.vendor.burn.jna.LibBootromByJNA;
import com.vendor.burn.jna.LibJTAGByJNA;
import com.vendor.burn.model.BoardConfig;
import com.vendor.burn.model.BurnContext;
import com.vendor.burn.model.ChipID;
import com.vendor.burn.model.FlashType;
import com.vendor.burn.model.JTAG_STATUS;
import com.vendor.burn.model.LocalConfig;
import com.vendor.burn.model.PartitionItem;
import com.vendor.burn.model.PlatformType;
import com.vendor.burn.model.ProductLine;
import com.vendor.burn.model.TransferWay;
import com.vendor.burn.model.config.BurnConfig;
import com.vendor.burn.model.config.PreferenceConfig;
import com.vendor.burn.model.define.IBurner;
import com.vendor.burn.model.devices.IFrameSetting;
import com.vendor.burn.model.devices.burners.AbstractBurner;
import com.vendor.burn.model.devices.burners.CommonBurner;
import com.vendor.burn.model.devices.unknown.UDFrameSetting;
import com.vendor.burn.model.devices.unknown.UnknownDevice;
import com.vendor.burn.model.event.EventType;
import com.vendor.burn.model.event.FastbootEvent;
import com.vendor.burn.model.receive.ReceiveTimeOutException;
import com.vendor.burn.model.struct.XmodelParameter;
import com.vendor.burn.preferences.FastbootPreferences;
import com.vendor.burn.serial.workers.AbstractBurnerWorker;
import com.vendor.burn.serial.workers.CallLibBootromWorker;
import com.vendor.burn.serial.workers.CallLibJTAGWorker;
import com.vendor.burn.serial.workers.CallLibUSBWorker;
import com.vendor.burn.serial.workers.Modem;
import com.vendor.burn.serial.workers.WorkerUtility;
import com.vendor.burn.transportWorks.AbstractTransportWorker;
import com.vendor.burn.transportWorks.NetTransportWorker;
import com.vendor.burn.transportWorks.SerialTransportWorker;
import com.vendor.burn.transportWorks.UsbTransportWorker;
import com.vendor.burn.utilitily.BufferedRandomAccessFile;
import com.vendor.burn.utilitily.FileUtils;
import com.vendor.burn.utilitily.GZipUtils;
import com.vendor.burn.utilitily.PropertiesUtility;
import com.vendor.burn.utilitily.ToolUtils;
import com.vendor.burn.views.FastbootView;
import com.vendor.platform.api.statistics.ToolStatistics;
import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.widgets.Display;
import org.vendor.tftp.prefences.TFTPPreferences;
import org.vendor.tftp.tool.TFTPService;

public class FastbootBurnerWorker extends AbstractBurnerWorker implements Modem {
  static final int TIMEOUT_XMODEM_EOT = 30000;
  
  static String msg_FailedToPrepareFile = "Failed to prepare File!";
  
  static String msg_FailedToLoadVersion = "Failed to load version info!";
  
  static String msg_BootBurnedSuccessfully = "Boot burned successfully.";
  
  private static long tempFileDivisionSize = 0L;
  
  protected static final String BLOCK_MARKED_BAD = "WARNING: block is marked as bad";
  
  protected String version;
  
  protected BoardConfig boardConfig;
  
  protected LocalConfig localConfig;
  
  protected String udpDdrFreeSizeStr;
  
  String ubootStartLog;
  
  protected static final int OUTPUT_PARAM_SIZE = 1;
  
  protected static final int TIMEMS = 10000;
  
  private static final int CANCEL_CODE = 256;
  
  public static final String WRITEBEFORE = "writeBefore";
  
  public static final String WRITEAFTER = "writeAfter";
  
  private static final String PARTITION_SBL = "sbl";
  
  protected TimeLogger burnBootTimeLogger = new TimeLogger();
  
  protected TimeLogger tftpTimeLogger = new TimeLogger();
  
  protected TimeLogger burnTimeLogger = new TimeLogger();
  
  protected TimeLogger waitZipTimeLogger = new TimeLogger();
  
  protected TimeLogger zipAndPageTimeLogger = new TimeLogger();
  
  protected TimeLogger unZipTimeLogger = new TimeLogger();
  
  protected TimeLogger eraseTimeLogger = new TimeLogger();
  
  protected TimeLogger spellingSparseFileLogger = new TimeLogger();
  
  protected TimeLogger crcLogger = new TimeLogger();
  
  protected TimeLogger i2cTimeLogger = new TimeLogger();
  
  protected TimeLogger flashTimeLogger = new TimeLogger();
  
  protected TimeLogger resetTimeLogger = new TimeLogger();
  
  protected static final int TFTP_MAX_RETURN_COUNT = 3;
  
  public FastbootBurnerWorker(IFrameSetting frameSetting, IBurner burner) {
    super(frameSetting);
    this.burner = burner;
    if (burner != null) {
      BurnContext burnContext = burner.getBurnContext();
      this.localConfig = burnContext.getLocalConfig();
      this.boardConfig = burnContext.getBoardConfig();
    } 
  }
  
  public boolean burn(IProgressMonitor uimonitor, FlashType flashType, String filePath) throws IOException, ReceiveTimeOutException {
    if (!ToolUtils.isCmdLine()) {
      ToolStatistics.INSTANCE.logUsage("BurnTool", "Burn By Fastboot", "ChipName: " + this.burner
          .getBurnContext().getDeviceSetting().getName());
      ToolStatistics.INSTANCE.logToolInUse("BurnTool");
      String versionInfo = getVersionInfo("Burn By Fastboot");
      CallLibJTAGWorker.printDebugInfo(versionInfo);
    } 
    try {
      setProgressMonitor(uimonitor);
      uimonitor.beginTask("", 100);
      registerJnaFunction(uimonitor, 100);
      FastbootEvent event = getMonitor().getEvent();
      event.init(EventType.BURN);
      setFlashType(flashType);
      TransferWay transferWay = this.burner.getBurnContext().getTransferWay();
      File file1 = new File(filePath);
      if (transferWay == TransferWay.Uart2Compress) {
        uimonitor.setTaskName("Init Board");
        ArrayList<FlashType> hasFlashTypes = new ArrayList<>();
        hasFlashTypes.add(flashType);
        boolean initBoard = initBoard(hasFlashTypes);
        if (!initBoard)
          return false; 
        getTempFileDivisionSize(transferWay);
      } else {
        if (transferWay == TransferWay.Usb2I2c || transferWay == TransferWay.I2CBootrom) {
          boolean result = burnSinglePartByI2c(uimonitor, file1, "0", flashType, true, ChipID.chip0);
          if (result) {
            event.setSuccess(true);
            uimonitor.done();
            return true;
          } 
          return false;
        } 
        if (transferWay == TransferWay.USBNonNude) {
          boolean result = false;
          result = bootromByNonNude(uimonitor);
          if (!result)
            return false; 
          result = downloadBootByUSB(uimonitor, file1, transferWay);
          if (!result)
            return false; 
        } else if (((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).isSupportBurnByLibBootrom()) {
          if (!programmerDownload(SubMonitor.convert(uimonitor, "", 86), filePath, ((AbstractBurner)this.burner)
              .getProgrammerFile(), null)) {
            returnError("Failed to download fastboot!");
            return false;
          } 
        } else if (this.burner.getBurnContext().getPlatformType() == PlatformType.NonCA) {
          if (!bootDownload(SubMonitor.convert(uimonitor, "", 86), filePath))
            return false; 
        } else if (!programmerDownload(SubMonitor.convert(uimonitor, "", 86), filePath, ((AbstractBurner)this.burner)
            .getProgrammerFile(), null)) {
          return false;
        } 
      } 
      if (flashType == FlashType.ddr) {
        try {
          Thread.sleep(2000L);
        } catch (InterruptedException e) {
          warningln("Error occurs while waiting DDR run.");
          return false;
        } 
        received("" + serialRecvBuf + serialRecvBuf);
        event.setSuccess(true);
        uimonitor.done();
        deleteContent(usbsReceived);
        return true;
      } 
      String unzipAddr = Long.toHexString(tempFileDivisionSize + getHexStartAddressForInt());
      if (this.burner.getBurnContext().getTransferWay() == TransferWay.Uart2Compress) {
        File partFile = copyComFileToToolPath(uimonitor, file1);
        if (partFile == null)
          return false; 
        File compressFile = GZipUtils.compress(uimonitor, partFile, false, true);
        if (compressFile == null) {
          returnError(NLS.bind(Messages.ERROR_INFO_DISK_SPACE_LOW, partFile));
          return false;
        } 
        downloadFileSerial(SubMonitor.convert(uimonitor, "", 86), compressFile, false);
        if (compressFile.exists())
          FileUtils.deleteFile(compressFile); 
        if (partFile.exists())
          FileUtils.deleteFile(partFile); 
        if (this.burner.getBurnContext().getTransferWay() == TransferWay.Uart2Compress) {
          UnZipCommand unZipCommand = UnZipCommand.unzip(getHexStartAddress(), unzipAddr);
          sendCommand(unZipCommand.toString(), unZipCommand
              .calculateTimeOut(Long.toHexString(tempFileDivisionSize)));
        } 
        if (flashType == FlashType.ddr)
          sendCommandNoReturnValue("go " + unzipAddr); 
      } 
      uimonitor.setTaskName("Loading Info");
      if (loadVersionInfo(transferWay)) {
        FlashType loadBootMode = loadBootMode();
        if (flashType != loadBootMode && loadBootMode != null) {
          returnError(NLS.bind(Messages.ERROR_INFO_Invalid_Mode_And_RealMode, loadBootMode.name()));
          return false;
        } 
      } else {
        warningln(msg_FailedToLoadVersion);
        return false;
      } 
      uimonitor.worked(1);
      uimonitor.setTaskName("Burning Boot");
      long blockSize = getBlockSize(flashType, ChipID.chip0, true);
      if (blockSize == 0L) {
        String error = "Failed to load blocksize for " + flashType.name();
        returnError(error);
        return false;
      } 
      long fileSize = file1.length();
      long eraseLength = fileSize;
      if (eraseLength % blockSize != 0L)
        eraseLength = (eraseLength / blockSize + 1L) * blockSize; 
      long writeLength = getWriteLength(flashType, blockSize, fileSize);
      if (!burnFastboot(SubMonitor.convert(uimonitor, "", 12), flashType, 
          Long.toHexString(eraseLength), Long.toHexString(writeLength), file1)) {
        returnError("Failed to burn boot.");
        return false;
      } 
      TransferWay curTransferWay = this.burner.getBurnContext().getTransferWay();
      boolean isResetJtag = ((FastbootView.isChipBranchSeries2() || FastbootView.isChipBranchSeries0()) && (curTransferWay == TransferWay.JTAG || curTransferWay == TransferWay.JTAGAndNet || curTransferWay == TransferWay.JTAGAndSerial));
      if (!isResetJtag)
        sendCommandNoReturnValue("reset"); 
      event.setSuccess(true);
      infoln(msg_BootBurnedSuccessfully);
    } finally {
      File file = new File(filePath);
      printCrcValue(file);
      uimonitor.done();
    } 
    return true;
  }
  
  protected boolean downloadBootByUDP(IProgressMonitor uimonitor, File file) {
    StringBuilder sendCommand = new StringBuilder("\nSend command");
    infoln("" + sendCommand + ":\tDownloading File : " + sendCommand);
    SubMonitor subProgress = SubMonitor.convert(uimonitor, "", 100);
    boolean result = CallLibBootromWorker.downloadData(CallLibBootromWorker.getUdpFd(), file.getPath(), 
        getHexStartAddressForInt());
    subProgress.worked(100);
    if (!result) {
      returnError(Messages.ERROR_INFO_UDP_DOWNLOAD_FAILED);
      return false;
    } 
    return true;
  }
  
  protected boolean downloadBootByUSB(IProgressMonitor uimonitor, File file, TransferWay transferWay) {
    infoln("\nSend command:\tDownloading File : " + file.getName());
    SubMonitor subProgress = SubMonitor.convert(uimonitor, "", 100);
    boolean result = CallLibUSBWorker.downloadFileByUsb(file, getHexStartAddressForInt(), transferWay);
    subProgress.worked(100);
    if (!result) {
      returnError(Messages.ERROR_INFO_USB_DOWNLOAD_FAILED);
      return false;
    } 
    return true;
  }
  
  public boolean bootBurn(SubMonitor subProgressMonitor, FlashType flashType, String eraseLength, String writeLength, ChipID chipID) throws IOException, ReceiveTimeOutException {
    String address = "0x0";
    String wLen = writeLength;
    if (wLen == null)
      wLen = eraseLength; 
    String ddrAddress = getHexStartAddress();
    String unzipAddr = Long.toHexString(tempFileDivisionSize + getHexStartAddressForInt());
    if (this.burner.getBurnContext().getTransferWay() == TransferWay.Uart2Compress)
      ddrAddress = unzipAddr; 
    FlashType flash = flashType;
    List<SerialCommand> commands = new ArrayList<>();
    if (FlashType.isEmmcOrUfs(flashType)) {
      commands.clear();
      MmcPartCommand.addMmcPartConfCommand("writeBefore", commands, flash);
      int commandType = ((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).getEmmcWriteCmdType();
      int emmcType = ((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).getEmmcType();
      if (this.burner.getBurnContext().getPlatformType() == PlatformType.NonCA || commandType == 0) {
        if (emmcType == 2) {
          commands.add(WriteCommand.writeEmmcOrUfs(flashType, ddrAddress, "0", wLen, this.burner));
        } else {
          commands.add(WriteCommand.writeeMMC(flashType, ddrAddress, "1", wLen));
        } 
      } else if (emmcType == 2) {
        commands.add(WriteCommand.writeEmmcOrUfs(flashType, ddrAddress, "0", wLen, this.burner));
      } else if (emmcType == 3) {
        commands.add(WriteCommand.writeeMMC(flashType, ddrAddress, "1", wLen));
      } else {
        commands.add(WriteCommand.bootWriteeMMC(ddrAddress, "0", wLen));
      } 
      MmcPartCommand.addMmcPartConfCommand("writeAfter", commands, flash);
    } else {
      if (flash == FlashType.spi)
        commands.add(SFCommand.getInstance(chipID)); 
      commands.add(EraseCommand.erase(flash, address, eraseLength));
      commands.add(WriteCommand.write(flash, ddrAddress, address, wLen));
    } 
    try {
      int total = commands.size();
      subProgressMonitor.beginTask("", total * 100);
      subProgressMonitor.setTaskName("Burning Boot");
      for (int i = 0; i < commands.size(); i++) {
        SerialCommand serialCommand = commands.get(i);
        subProgressMonitor.subTask("Execute command:" + serialCommand.getCommand());
        if (!sendCommand(serialCommand))
          return false; 
        subProgressMonitor.worked(100);
      } 
    } finally {
      subProgressMonitor.done();
    } 
    return true;
  }
  
  private boolean burnFastboot(SubMonitor subProgressMonitor, FlashType flashType, String eraseLength, String writeLength, File file) throws IOException, ReceiveTimeOutException {
    if (FastbootView.isSupportSerial() || FastbootView.isChipBranchSeries10()) {
      TransferWay transferWay = this.burner.getBurnContext().getTransferWay();
      if (isNeedSetupNet(transferWay)) {
        if (!setupNetwork(transferWay)) {
          returnError("Failed to set net work!");
          return false;
        } 
        switchTFTPDownloadHome(file);
      } 
      subProgressMonitor.setTaskName("Sending MW command.");
      MWCommand memsetCommand = MWCommand.mwAddress(getHexStartAddress(), writeLength);
      if (!sendCommand((SerialCommand)memsetCommand))
        return false; 
      XmodelParameter xmodelPar = new XmodelParameter(file.length(), 0L);
      if (!downloadFileByTrans(subProgressMonitor, xmodelPar, file, 95))
        return false; 
    } 
    if (!bootBurn(SubMonitor.convert(this.uimonitor, "", 12), flashType, eraseLength, writeLength, ChipID.chip0)) {
      returnError("Failed to burn boot.");
      return false;
    } 
    return true;
  }
  
  private boolean downloadFileByTrans(SubMonitor subProgressMonitor, XmodelParameter xmodelPar, File file, int progressWork) throws IOException, ReceiveTimeOutException {
    SerialTransportWorker serialTransportWorker;
    NetTransportWorker netTransportWorker;
    UsbTransportWorker usbTransportWorker;
    TransferWay transferWay = this.burner.getBurnContext().getTransferWay();
    AbstractTransportWorker abstractTransportWorker = null;
    switch (null.$SwitchMap$com$vendor$burn$model$TransferWay[transferWay.ordinal()]) {
      case 1:
        serialTransportWorker = new SerialTransportWorker(this.frameSetting, this.burner);
        break;
      case 2:
        netTransportWorker = new NetTransportWorker(this.frameSetting, this.burner);
        break;
      case 3:
        usbTransportWorker = new UsbTransportWorker(this.frameSetting, this.burner);
        break;
    } 
    if (usbTransportWorker == null) {
      returnError("New TransportWorker failed.");
      return false;
    } 
    return usbTransportWorker.downloadFileByTransport(subProgressMonitor, xmodelPar, file, progressWork);
  }
  
  private boolean isJTAG() {
    TransferWay transferWay = this.burner.getBurnContext().getTransferWay();
    boolean isBootromByJTAG = (transferWay == TransferWay.JTAGAndNet || transferWay == TransferWay.JTAG || transferWay == TransferWay.JTAGAndSerial);
    return isBootromByJTAG;
  }
  
  public boolean bootDownload(String filePath) throws IOException {
    TransferWay transferWay = this.burner.getBurnContext().getTransferWay();
    boolean isSupportLibBootrom = (((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).isSupportBurnByLibBootrom() && transferWay != TransferWay.JTAGAndNet && transferWay != TransferWay.JTAG && transferWay != TransferWay.JTAGAndSerial);
    boolean isBootromByJTAG = (transferWay == TransferWay.JTAGAndNet || transferWay == TransferWay.JTAG || transferWay == TransferWay.JTAGAndSerial);
    SubMonitor subMonitor = null;
    if (this.uimonitor != null)
      subMonitor = SubMonitor.convert(this.uimonitor, "", 50); 
    boolean result = false;
    if (isSupportLibBootrom) {
      result = burnByLibBootrom(subMonitor, filePath);
    } else if (isBootromByJTAG) {
      deleteContent(jtagReceived);
      FastbootView.setFastbootStart(false);
      result = burnByJTAG(filePath);
      if (result)
        infoln("Boot started successfully!"); 
    } else {
      try {
        new CallLibBootromWorker(this.frameSetting, this.burner, subMonitor);
        LibBootromByJNA.instance.RegisterPrint(CallLibBootromWorker.callback);
        LibBootromByJNA.instance.RegisterCancel(CallLibBootromWorker.callbackCancel);
        if (!downloadFile())
          return false; 
        infoln("Boot download completed!");
        if (getFlashType() != DDR)
          infoln("Boot started successfully!"); 
        result = true;
      } finally {
        if (this.fileList != null)
          this.fileList = null; 
      } 
    } 
    if (result)
      if (CommonBurner.isSocketTransfer) {
        Socket socketConnection = ((AbstractBurner)this.burner).createSocket(getMonitor(), this);
        if (socketConnection == null)
          return false; 
      }  
    return result;
  }
  
  protected boolean tftpAndCheckCRC(SubMonitor subProgressMonitor, File file, boolean openTftpCrcCheck) throws IOException, ReceiveTimeOutException {
    boolean result = false;
    if (TFTPService.TFTPSERVICE.getServerDirectory() != null && 
      !TFTPService.TFTPSERVICE.getServerDirectory().equals(file.getParentFile()))
      switchTFTPDownloadHome(file); 
    if (this.tftpTimeLogger != null)
      this.tftpTimeLogger.logStart(file.getName(), file.length()); 
    if (subProgressMonitor != null)
      subProgressMonitor.setTaskName("Download File : " + file.getName()); 
    TFTPCommand downloadCommand = TFTPCommand.download(file, getHexStartAddress());
    result = sendCommand((SerialCommand)downloadCommand);
    if (this.tftpTimeLogger != null)
      this.tftpTimeLogger.logEnd(); 
    if (result && 
      openTftpCrcCheck)
      result = checkTFTPCrc32(subProgressMonitor, file); 
    return result;
  }
  
  private boolean checkTFTPCrc32(SubMonitor subProgressMonitor, File file) throws IOException, ReceiveTimeOutException {
    long fileCrcValue = CrcCommand.calculateCRC(file);
    boolean result = false;
    if (this.crcLogger != null)
      this.crcLogger.logStart(file.getName(), file.length()); 
    if (sendCommand((SerialCommand)CrcCommand.sendCRCCommand(file, getHexStartAddress()))) {
      String crcReturnStr = getCurReceived();
      result = CrcCommand.compareFileCRC(fileCrcValue, CrcCommand.receiveCRC(crcReturnStr));
      if (!result) {
        boolean isSupportTFTPWin = ((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).isSupportTFTPWin();
        if (isSupportTFTPWin) {
          boolean isCheck = false;
          if (ToolUtils.isCmdLine()) {
            isCheck = PreferenceConfig.isTftpWinSizeOpen();
          } else {
            isCheck = FastbootPreferences.isOpenTftpWindowSize();
          } 
          if (isCheck && 
            TFTPService.getWinSizeBlockCount() > 0 && TFTPService.getWinSizeBlockCount() > 1) {
            TFTPService.setWinSizeBlockCount(TFTPService.getWinSizeBlockCount() - 1);
            infoln("The current Window Size value is " + TFTPService.getWinSizeBlockCount() + 1);
          } 
        } else {
          TFTPService.setWinSizeBlockCount(1);
        } 
        warningln("Warn:CRC check failed, TFTP download again.");
      } 
    } else {
      warningln("Warn:CRC send command failed, TFTP download again.");
    } 
    if (this.crcLogger != null)
      this.crcLogger.logEnd(); 
    return result;
  }
  
  protected boolean burnByLibBootrom(SubMonitor subProgressMonitor, String filePath) {
    try {
      new CallLibBootromWorker(this.frameSetting, this.burner, subProgressMonitor);
      LibBootromByJNA.instance.RegisterPrint(CallLibBootromWorker.callback);
      LibBootromByJNA.instance.RegisterCancel(CallLibBootromWorker.callbackCancel);
      if (ToolUtils.isCmdLine())
        CallLibBootromWorker.setTypeFrameFrequency(this.burner.getBurnContext().getDwTimeout()); 
      TransferWay transferWay = this.burner.getBurnContext().getTransferWay();
      if (transferWay == TransferWay.NetNonNude)
        filePath = null; 
      if (UnknownDevice.isOpenLightDebug())
        initLightState(); 
      infoln(getStartMessage());
      monitorProgress(subProgressMonitor, 1);
      boolean result = false;
      CallLibBootromWorker.setChipType(FastbootView.getDepartmentNumber(this.burner), 
          FastbootView.getChipType(this.burner));
      if (!openTransmissionMode(transferWay, this.boardConfig))
        return false; 
      int ddrOrFlash = (getFlashType() != DDR) ? 0 : 1;
      int[] caType = new int[1];
      int isReboot = (!ToolUtils.isCmdLine() && FastbootPreferences.isResetBeforeBurn()) ? 1 : 0;
      long[] initDdrAddr = new long[1];
      CallLibBootromWorker.setReboot(isReboot);
      result = CallLibBootromWorker.bootDownloadByLib(CallLibBootromWorker.getFd(), ddrOrFlash, filePath, caType, initDdrAddr);
      if (result) {
        String caTypeStr = "";
        if (caType[0] == 1) {
          caTypeStr = "CA";
          this.isCaChipByLibBootrom = true;
        } else {
          caTypeStr = "noCA";
          this.isCaChipByLibBootrom = false;
        } 
        updateCaTypeLbl(caTypeStr);
        ((UDFrameSetting)this.frameSetting).setDdrInitAddrByLibBootrom(initDdrAddr[0]);
      } else {
        return false;
      } 
      monitorProgress(subProgressMonitor, 100);
      return result;
    } finally {
      if (subProgressMonitor != null)
        subProgressMonitor.done(); 
    } 
  }
  
  private void monitorProgress(SubMonitor subProgressMonitor, int progress) {
    if (subProgressMonitor != null)
      subProgressMonitor.worked(progress); 
  }
  
  protected boolean burnByJTAG(SubMonitor subProgressMonitor, String filePath, int level) {
    try {
      String commName = this.burner.getBurnContext().getLocalConfig().getComPort();
      int jtagChipType = FastbootView.getJtagChipTypeByChipName().ordinal();
      int ddrOrFlash = (getFlashType() != DDR) ? 0 : 1;
      int[] errorCode = new int[1];
      long[] initDdrAddr = new long[1];
      if (FastbootView.isChipBranchSeries0())
        infoln("\n" + Messages.ERROR_INFO_Start_JTAG); 
      subProgressMonitor.worked(1);
      if (!sendCmdNoReturn())
        return false; 
      boolean result = CallLibJTAGWorker.bootDownloadByLib(jtagChipType, commName, ddrOrFlash, initDdrAddr, filePath, LibJTAGByJNA.JTAG_DLL_PATH, errorCode, level);
      if (!result) {
        checkJTAGReturn(errorCode);
        return result;
      } 
      if (FastbootView.isChipBranchSeries7())
        FastbootView.setFastbootStart(true); 
      ((UDFrameSetting)this.frameSetting).setDdrInitAddrByLibBootrom(initDdrAddr[0]);
      subProgressMonitor.worked(100);
      return result;
    } finally {
      subProgressMonitor.done();
    } 
  }
  
  private void checkJTAGReturn(int[] errorCode) {
    int error = errorCode[0] & 0xFFFF;
    if (error == 256) {
      handleCancel();
    } else if (error == JTAG_STATUS.OPENOCD_DEVICE_NOT_FOUND.getErrorCode()) {
      monitor.getEvent().setMessage(Messages.ERROR_JTAG_OPENOCD_DEVICE_NOT_FOUND);
    } else if (error == JTAG_STATUS.SERAIL_PORT_OPEN_FAILED.getErrorCode()) {
      monitor.getEvent().setMessage(Messages.SERAIL_PORT_OPEN_FAILED);
    } else if (error == JTAG_STATUS.SERAIL_PORT_NO_DATA.getErrorCode()) {
      monitor.getEvent().setMessage(Messages.SERAIL_PORT_NO_DATA);
    } else if (error == JTAG_STATUS.SERAIL_PORT_BOOT_NOT_RUN.getErrorCode()) {
      monitor.getEvent().setMessage(Messages.SERAIL_PORT_BOOT_NOT_RUN);
    } else if (error == JTAG_STATUS.BOOT_NOT_RUN.getErrorCode()) {
      monitor.getEvent().setMessage(Messages.BOOT_NOT_RUN);
    } else if (error == JTAG_STATUS.INVALID_PARAMETER.getErrorCode()) {
      monitor.getEvent().setMessage(Messages.INVALID_PARAMETER);
    } else if (error == JTAG_STATUS.WRONG_CHIP_ID.getErrorCode()) {
      monitor.getEvent().setMessage(Messages.WRONG_ID);
    } else if (error == JTAG_STATUS.INVALID_BASIC.getErrorCode()) {
      monitor.getEvent().setMessage(Messages.INVALID_BASIC);
    } else if (error == JTAG_STATUS.DOWNLOAD_FILE_FAILED.getErrorCode()) {
      monitor.getEvent().setMessage(Messages.DOWNLOAD_FILE_FAILED);
    } else if (error == JTAG_STATUS.READ_REG_ERROR.getErrorCode()) {
      monitor.getEvent().setMessage(Messages.READ_REG_ERROR);
    } else if (error == JTAG_STATUS.OPERATE_TIME_OUT.getErrorCode()) {
      monitor.getEvent().setMessage(Messages.OPERATE_TIME_OUT);
    } else if (error == JTAG_STATUS.INIT_OPENOCD_FAIL.getErrorCode()) {
      monitor.getEvent().setMessage(Messages.INIT_OPENOCD_FAIL);
    } else if (error == JTAG_STATUS.INIT_DDR_FAIL.getErrorCode()) {
      monitor.getEvent().setMessage(Messages.INIT_DDR_FAIL);
    } else if (error == JTAG_STATUS.CRC32_VERIFY_TIMEOUT.getErrorCode()) {
      monitor.getEvent().setMessage(Messages.CRC32_VERIFY_TIMEOUT);
    } else if (error == JTAG_STATUS.BOARD_POWER_OFF.getErrorCode()) {
      monitor.getEvent().setMessage(Messages.BOARD_POWER_OFF);
    } 
  }
  
  protected boolean burnByJTAG(String filePath) {
    if (filePath != null && filePath.isEmpty())
      filePath = null; 
    String commName = this.burner.getBurnContext().getLocalConfig().getComPort();
    int jtagChipType = FastbootView.getJtagChipTypeByChipName().ordinal();
    int ddrOrFlash = (getFlashType() != DDR) ? 0 : 1;
    int[] errorCode = new int[1];
    long[] initDdrAddr = new long[1];
    if (FastbootView.isChipBranchSeries0())
      infoln("\n" + Messages.ERROR_INFO_Start_JTAG); 
    if (!sendCmdNoReturn())
      return false; 
    int jtagFrequency = this.burner.getBurnContext().getJtagFrequency();
    boolean result = CallLibJTAGWorker.bootDownloadByLib(jtagChipType, commName, ddrOrFlash, initDdrAddr, filePath, LibJTAGByJNA.JTAG_DLL_PATH, errorCode, jtagFrequency);
    if (!result) {
      checkJTAGReturn(errorCode);
      return result;
    } 
    if (FastbootView.isChipBranchSeries7())
      FastbootView.setFastbootStart(true); 
    ((UDFrameSetting)this.frameSetting).setDdrInitAddrByLibBootrom(initDdrAddr[0]);
    return result;
  }
  
  private boolean sendCmdNoReturn() {
    if (FastbootView.getJtagChipTypeByChipName() != LibJTAGByJNA.EJtagChipType.s7 || !BurnConfig.isSendUartCommand())
      return true; 
    SubMonitor subProgressMonitor = SubMonitor.convert(this.uimonitor, "", 100);
    String comPort = this.burner.getBurnContext().getLocalConfig().getComPort();
    infoln("Open Uart: " + comPort);
    if (!CallLibBootromWorker.openSerialComm(comPort, 115200)) {
      infoln("CreateSerialComm failed!");
      return false;
    } 
    new CallLibBootromWorker(this.frameSetting, this.burner, subProgressMonitor);
    LibBootromByJNA.instance.RegisterPrint(CallLibBootromWorker.callback);
    LibBootromByJNA.instance.RegisterCancel(CallLibBootromWorker.callbackCancel);
    infoln("Send Uart Command");
    int[] pCmd = { 
        222, 173, 190, 235, 222, 173, 190, 235, 222, 173, 
        190, 235 };
    CallLibBootromWorker.sendBytes(CallLibBootromWorker.getComFd(), 
        convertToByte(pCmd), pCmd.length);
    infoln("Close Uart");
    CallLibBootromWorker.destroyDevice(CallLibBootromWorker.getComFd());
    return true;
  }
  
  private boolean setupUdpPort() throws IOException, ReceiveTimeOutException {
    int port = this.burner.getBurnContext().getBoardConfig().getUdpPort();
    NetCommand command = NetCommand.setServerPort(String.valueOf(port));
    if (!sendCommand(command.getCommand(), command.getTimeout()))
      return false; 
    if (!sendCommand("saveenv", command.getTimeout()))
      return false; 
    return true;
  }
  
  protected boolean startFastbootUdpTransferWay(IProgressMonitor uimonitor, TransferWay curTransferWay) throws IOException {
    if (curTransferWay == TransferWay.UDP && ToolUtils.isCmdLine()) {
      try {
        this.udpDdrFreeSizeStr = getDDRFreeParamBySerial();
      } catch (ReceiveTimeOutException e) {
        return false;
      } 
      if (this.udpDdrFreeSizeStr == null || this.udpDdrFreeSizeStr.isEmpty())
        return false; 
      try {
        NetCommand[] commands = buildNetCommands(curTransferWay);
        if (commands == null)
          return false; 
        for (NetCommand netCommand : commands) {
          if (!sendCommand(netCommand.getCommand(), netCommand.getTimeout()))
            return false; 
        } 
        if (!setupUdpPort())
          return false; 
      } catch (ReceiveTimeOutException e) {
        return false;
      } 
      String ddrFreeSize = getDDRFreeParam("size:", this.udpDdrFreeSizeStr, TransferWay.Serial);
      String ddrFreeBaseAddr = getDDRFreeParam("baseaddr:", this.udpDdrFreeSizeStr, TransferWay.Serial);
      ((UDFrameSetting)this.frameSetting).setDdrInitAddrByLibBootrom(parseLong(ddrFreeBaseAddr));
      String command = INFOCommand.getUDPServiceCommand(ddrFreeBaseAddr, ddrFreeSize).getCommand();
      try {
        sendCommandDontCareEOTBySerial(command, TIMEOUT_SEND_FRAME);
      } catch (IOException iOException) {
      
      } catch (ReceiveTimeOutException receiveTimeOutException) {}
      if (!CallLibBootromWorker.createUdpSocket(this.burner.getBurnContext().getBoardConfig().getIp(), this.burner
          .getBurnContext().getBoardConfig().getUdpPort()))
        return false; 
    } 
    return true;
  }
  
  protected boolean startFastbootUsbTransferWay(IProgressMonitor uimonitor, TransferWay curTransferWay) throws IOException {
    if (curTransferWay == TransferWay.UsbAndSerial) {
      String ddrFreeSizeStr = null;
      try {
        ddrFreeSizeStr = getDDRFreeParamBySerial();
        receivedln(ddrFreeSizeStr);
      } catch (ReceiveTimeOutException receiveTimeOutException) {}
      if (ddrFreeSizeStr == null || ddrFreeSizeStr.isEmpty())
        return false; 
      String command = null;
      if (ToolUtils.isCmdLine()) {
        command = INFOCommand.getUsbSerial(BurnConfig.getUsbSerialNumber()).getCommand();
        infoln("\nSend command:\t" + command);
        if (!serialCommand(command, TIMEOUT_SEND_FRAME))
          return false; 
        try {
          Thread.sleep(1000L);
        } catch (InterruptedException interruptedException) {}
      } 
      String ddrFreeSize = getDDRFreeParam("size:", ddrFreeSizeStr, TransferWay.Serial);
      String ddrFreeBaseAddr = getDDRFreeParam("baseaddr:", ddrFreeSizeStr, TransferWay.Serial);
      ((UDFrameSetting)this.frameSetting).setDdrInitAddrByLibBootrom(parseLong(ddrFreeBaseAddr));
      command = INFOCommand.getFastboot(ddrFreeBaseAddr, ddrFreeSize).getCommand();
      infoln("\nSend command:\t" + command);
      if (!serialCommandFastboot(command, TIMEOUT_SEND_FRAME)) {
        debugPrintln("Failed to send command frame");
        return false;
      } 
      try {
        Thread.sleep(2000L);
      } catch (InterruptedException interruptedException) {}
      boolean openUsbResult = CallLibUSBWorker.openUsb(uimonitor, 3000L, TransferWay.UsbAndSerial, 
          BurnConfig.getUsbSerialNumber());
      if (!openUsbResult)
        return false; 
    } 
    return true;
  }
  
  public boolean bootDownload(SubMonitor subProgressMonitor, String filePath) throws IOException {
    subProgressMonitor.beginTask("", 100);
    TransferWay transferWay = this.burner.getBurnContext().getTransferWay();
    boolean isSupportLibBootrom = (((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).isSupportBurnByLibBootrom() && transferWay != TransferWay.JTAGAndNet && transferWay != TransferWay.JTAG && transferWay != TransferWay.JTAGAndSerial);
    boolean isBootromByJTAG = (transferWay == TransferWay.JTAGAndNet || transferWay == TransferWay.JTAG || transferWay == TransferWay.JTAGAndSerial);
    if (isSupportLibBootrom)
      try {
        subProgressMonitor.setTaskName("Downloading Boot");
        if (!burnByLibBootrom(SubMonitor.convert(this.uimonitor, "", 98), filePath)) {
          updateLightState();
          return false;
        } 
        infoln("Boot started successfully!");
        if (CommonBurner.isSocketTransfer) {
          Socket socketConnection = ((AbstractBurner)this.burner).createSocket(getMonitor(), this);
          if (socketConnection == null)
            return false; 
        } 
        if (!startFastbootUdpTransferWay(this.uimonitor, this.burner.getBurnContext().getTransferWay()))
          return false; 
        if (!startFastbootUsbTransferWay(this.uimonitor, this.burner.getBurnContext().getTransferWay()))
          return false; 
        return true;
      } finally {
        subProgressMonitor.done();
      }  
    if (isBootromByJTAG) {
      deleteContent(jtagReceived);
      FastbootView.setFastbootStart(false);
      int jtagFrequency = this.burner.getBurnContext().getJtagFrequency();
      try {
        subProgressMonitor.setTaskName("Downloading Boot");
        if (!burnByJTAG(SubMonitor.convert(this.uimonitor, "", 98), filePath, jtagFrequency))
          return false; 
        infoln("Boot started successfully!");
        if (transferWay != TransferWay.JTAGAndNet)
          FastbootView.setFastbootStart(true); 
        return true;
      } finally {
        subProgressMonitor.done();
      } 
    } 
    try {
      subProgressMonitor.setTaskName("Downloading Boot");
      new CallLibBootromWorker(this.frameSetting, this.burner, subProgressMonitor);
      LibBootromByJNA.instance.RegisterPrint(CallLibBootromWorker.callback);
      LibBootromByJNA.instance.RegisterCancel(CallLibBootromWorker.callbackCancel);
      if (!downloadFile(filePath))
        return false; 
      if (getFlashType() != DDR)
        infoln("Boot started successfully!"); 
      return true;
    } finally {
      if (this.fileList != null)
        this.fileList = null; 
      subProgressMonitor.done();
    } 
  }
  
  public boolean programmerDownload(String bootFile, String programmerFile) {
    try {
      if (((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).isSupportBurnByLibBootrom()) {
        boolean bootAndProgrammerisInVaild = (((bootFile == null || bootFile.isEmpty()) && (programmerFile == null || programmerFile.isEmpty())) || this.burner.getBurnContext().getTransferWay() == TransferWay.NetNonNude);
        if (bootAndProgrammerisInVaild) {
          if (!bootDownload(null))
            return false; 
        } else {
          boolean programmerInvaild = (programmerFile == null || programmerFile.isEmpty());
          if (programmerInvaild) {
            if (!bootDownload(bootFile))
              return false; 
            if (!this.isCaChipByLibBootrom)
              return true; 
          } else if (!bootDownload(programmerFile)) {
            return false;
          } 
        } 
      } else if (isJTAG()) {
        boolean programmerInvaild = (programmerFile == null || programmerFile.isEmpty());
        if (programmerInvaild) {
          if (!bootDownload(bootFile))
            return false; 
          if (!this.isCaChipByLibBootrom)
            return true; 
        } else if (!bootDownload(programmerFile)) {
          return false;
        } 
      } else {
        boolean bootAndProgisInVaild = ((bootFile == null || bootFile.isEmpty()) && (programmerFile == null || programmerFile.isEmpty()));
        if (!bootAndProgisInVaild) {
          if (!downloadFile())
            return false; 
          if (getFlashType() == DDR)
            return true; 
        } 
        if (!waitBootRunCA())
          return false; 
      } 
    } catch (IOException e) {
      return false;
    } 
    if (!downloadBootAgain(bootFile))
      return false; 
    return true;
  }
  
  private boolean downloadBootAgain(String bootFile) {
    boolean bootFileVaild = (bootFile != null && !bootFile.isEmpty());
    if (bootFileVaild) {
      File bFile = new File(bootFile);
      boolean isSerialOnly = ((this.burner instanceof com.vendor.burn.model.devices.IBootBurner && !(this.burner instanceof com.vendor.burn.model.devices.IAddressBurner)) || this.burner instanceof com.vendor.burn.model.define.IChecker);
      boolean downloadResult = true;
      TransferWay transferWay = this.burner.getBurnContext().getTransferWay();
      boolean needOpenNetWork = isNeedOpenNetWork(isSerialOnly, transferWay);
      if (needOpenNetWork) {
        try {
          if (!setupNetwork(transferWay)) {
            returnError("Failed to setup network!");
            return false;
          } 
          switchTFTPDownloadHome(bFile);
          if (!sendCommand((SerialCommand)TFTPCommand.download(bFile, getHexStartAddress())))
            downloadResult = false; 
        } catch (IOException e) {
          downloadResult = false;
        } catch (ReceiveTimeOutException e) {
          downloadResult = false;
        } 
      } else if (transferWay == TransferWay.JTAG || transferWay == TransferWay.JTAGAndSerial) {
        boolean result = false;
        try {
          result = jtagAndCheckCRC(bFile);
        } catch (IOException iOException) {
        
        } catch (ReceiveTimeOutException receiveTimeOutException) {}
        if (!result) {
          returnError(Messages.FAILED_TO_DOWNLOAD_FILE_BY_JTAG);
          return false;
        } 
      } else {
        new CallLibBootromWorker(this.frameSetting, this.burner, 
            SubMonitor.convert(this.uimonitor, "", 100));
        LibBootromByJNA.instance.RegisterPrint(CallLibBootromWorker.callback);
        LibBootromByJNA.instance.RegisterCancel(CallLibBootromWorker.callbackCancel);
        try {
          if (!downloadFileSerial(bFile))
            downloadResult = false; 
        } catch (IOException e) {
          downloadResult = false;
        } catch (ReceiveTimeOutException e) {
          downloadResult = false;
        } 
      } 
      if (!downloadResult) {
        warningln("\nFailed to downlaod file. Please check the board status and connection, then try again.");
        return false;
      } 
    } 
    return true;
  }
  
  private boolean isNeedOpenNetWork(boolean isSerialOnly, TransferWay transferWay) {
    boolean needOpenNetWork = ((transferWay == TransferWay.Net || transferWay == TransferWay.UsbTransNet || transferWay == TransferWay.JTAGAndNet) && !isSerialOnly);
    return needOpenNetWork;
  }
  
  public boolean programmerDownload(SubMonitor subProgressMonitor, String bootFile, String programmerFile, List<PartitionItem> parts) {
    subProgressMonitor.setTaskName("Downloading Boot");
    registerJnaFunction((IProgressMonitor)subProgressMonitor, 100);
    try {
      boolean isLibBootromOrJtag = (((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).isSupportBurnByLibBootrom() || this.burner.getBurnContext().getTransferWay() == TransferWay.JTAGAndNet || this.burner.getBurnContext().getTransferWay() == TransferWay.JTAG || this.burner.getBurnContext().getTransferWay() == TransferWay.JTAGAndSerial);
      if (isLibBootromOrJtag) {
        boolean bootAndProgrammerisInVaild = (((bootFile == null || bootFile.isEmpty()) && (programmerFile == null || programmerFile.isEmpty())) || this.burner.getBurnContext().getTransferWay() == TransferWay.NetNonNude);
        if (bootAndProgrammerisInVaild) {
          if (!bootDownload(subProgressMonitor, null))
            return false; 
        } else {
          boolean programmerInvaild = (programmerFile == null || programmerFile.isEmpty());
          if (programmerInvaild) {
            if (!bootDownload(subProgressMonitor, bootFile))
              return false; 
            if (!this.isCaChipByLibBootrom)
              return true; 
          } else {
            boolean bool = getNeedDownLoadSBL(parts);
            if (bool && bootFile != null && !bootFile.isEmpty()) {
              if (!bootDownload(subProgressMonitor, bootFile))
                return false; 
            } else if (!bootDownload(subProgressMonitor, programmerFile)) {
              return false;
            } 
          } 
        } 
      } else {
        boolean bootAndProgisInVaild = ((bootFile == null || bootFile.isEmpty()) && (programmerFile == null || programmerFile.isEmpty()));
        if (!bootAndProgisInVaild) {
          new CallLibBootromWorker(this.frameSetting, this.burner, subProgressMonitor);
          LibBootromByJNA.instance.RegisterPrint(CallLibBootromWorker.callback);
          LibBootromByJNA.instance.RegisterCancel(CallLibBootromWorker.callbackCancel);
          subProgressMonitor.subTask("Downloading boot");
          infoln("[Download File by Serial]\nfile = " + bootFile + "\naddress = " + getHexStartAddress());
          infoln("Serial Download Command = " + bootFile);
          if (!downloadFile(bootFile))
            return false; 
          if (getFlashType() == DDR)
            return true; 
        } 
        subProgressMonitor.subTask("Waiting Boot to Run");
        if (!waitBootRunCA())
          return false; 
      } 
    } catch (IOException e) {
      return false;
    } 
    if (getFlashType() == DDR)
      return true; 
    boolean needDownLoadSBL = getNeedDownLoadSBL(parts);
    if (needDownLoadSBL)
      return true; 
    subProgressMonitor.subTask("Preparing");
    if (!downloadBootAgain(subProgressMonitor, bootFile))
      return false; 
    return true;
  }
  
  protected boolean burnSBL(IProgressMonitor uiMonitor, List<PartitionItem> parts, String programmerPath) throws IOException, ReceiveTimeOutException {
    boolean needDownLoadSBL = getNeedDownLoadSBL(parts);
    if (needDownLoadSBL) {
      File sblFile = getSBLFile(parts);
      if (programmerPath != null && !programmerPath.isEmpty())
        sblFile = new File(programmerPath); 
      if (sblFile == null)
        return false; 
      boolean result = downloadFileSerial(SubMonitor.convert(this.uimonitor, "", 100), sblFile, true);
      if (!result)
        return result; 
      if (!waitBootRun(10000))
        return false; 
      infoln("SBL Started successfully!");
      PartitionItem fastboot = parts.get(0);
      if (!fastboot.isChecked())
        return true; 
      File file = fastboot.getFile();
      if (file == null)
        return false; 
      String bootFilePath = file.getPath();
      if (!downloadBootAgain(SubMonitor.convert(this.uimonitor, "", 95), bootFilePath))
        return false; 
    } 
    return true;
  }
  
  protected boolean getNeedDownLoadSBL(List<PartitionItem> parts) {
    if (!FastbootView.isNeedDownLoadSBLSeries() || parts == null)
      return false; 
    boolean hasSBL = false;
    boolean hasSBLFile = false;
    for (PartitionItem partitionItem : parts) {
      String name = partitionItem.getName();
      if (name != null && name.equalsIgnoreCase("sbl")) {
        hasSBL = true;
        File sblFile = partitionItem.getFile();
        if (sblFile != null) {
          boolean vaildFile = (sblFile.exists() && !sblFile.isDirectory());
          if (vaildFile) {
            hasSBLFile = true;
            break;
          } 
          MessageDialog.openWarning(Display.getCurrent().getActiveShell(), "BurnTool", Messages.WARN_SBL_IS_NOT_SELECT_FILE);
          break;
        } 
      } 
    } 
    return (hasSBL && hasSBLFile);
  }
  
  private File getSBLFile(List<PartitionItem> parts) {
    for (PartitionItem partitionItem : parts) {
      String name = partitionItem.getName();
      if (name.equalsIgnoreCase("sbl"))
        return partitionItem.getFile(); 
    } 
    return null;
  }
  
  private boolean downloadBootAgain(SubMonitor subProgressMonitor, String bootFile) {
    boolean bootFileVaild = (bootFile != null && !bootFile.isEmpty());
    if (bootFileVaild) {
      File bFile = new File(bootFile);
      boolean downloadResult = true;
      TransferWay transferWay = this.burner.getBurnContext().getTransferWay();
      boolean needOpenNetWork = isNeedSetupNet(transferWay);
      if (needOpenNetWork) {
        try {
          if (!setDdrAndNet(transferWay))
            return false; 
          switchTFTPDownloadHome(bFile);
          boolean result = false;
          int retransmitCount = 0;
          result = checkTftpCrcIfTrue(subProgressMonitor, bFile, result, retransmitCount);
          if (!result) {
            returnError(Messages.Partition_TFTP_CRC_Failed);
            return false;
          } 
        } catch (IOException e) {
          downloadResult = false;
        } catch (ReceiveTimeOutException e) {
          downloadResult = false;
        } 
      } else if (transferWay == TransferWay.JTAGAndSerial || transferWay == TransferWay.JTAG || transferWay == TransferWay.JTAGAndNet) {
        boolean result = false;
        try {
          result = jtagAndCheckCRC(bFile);
        } catch (IOException iOException) {
        
        } catch (ReceiveTimeOutException receiveTimeOutException) {}
        if (!result) {
          returnError(Messages.Partition_JTAG_CRC_Failed);
          return false;
        } 
      } else if (transferWay == TransferWay.UsbAndSerial || 
        FastbootView.isSupportUsbNonNude(transferWay)) {
        downloadResult = downloadBootByUSB((IProgressMonitor)subProgressMonitor, bFile, transferWay);
      } else if (transferWay == TransferWay.UDP) {
        downloadResult = downloadBootByUDP((IProgressMonitor)subProgressMonitor, bFile);
      } else {
        try {
          if (!downloadFileSerial(SubMonitor.convert(this.uimonitor, "", 20), bFile, false))
            downloadResult = false; 
        } catch (IOException e) {
          downloadResult = false;
        } catch (ReceiveTimeOutException e) {
          downloadResult = false;
        } 
      } 
      if (!downloadResult) {
        warningln("\nFailed to downlaod file. Please check the board status and connection, then try again.");
        return false;
      } 
    } 
    return true;
  }
  
  private boolean setDdrAndNet(TransferWay transferWay) {
    try {
      if (!setupNetwork(transferWay)) {
        returnError("Failed to setup network!");
        return false;
      } 
    } catch (ReceiveTimeOutException e) {
      returnError("Failed to setup network!");
      return false;
    } 
    return true;
  }
  
  public boolean checkTftpCrcIfTrue(SubMonitor subProgressMonitor, File bFile, boolean result, int retransmitCount) throws IOException, ReceiveTimeOutException {
    boolean openTftpCrcCheck = true;
    boolean checkResult = result;
    int retransmitMaxCount = retransmitCount;
    if (ToolUtils.isCmdLine()) {
      openTftpCrcCheck = (PreferenceConfig.isTftpCrcCheck() && !FastbootView.isChipBranchSeries7());
    } else {
      openTftpCrcCheck = FastbootPreferences.isOpenTftpCrcCheck();
    } 
    if (openTftpCrcCheck) {
      while (retransmitMaxCount < 3) {
        infoln("tftp crc count:" + retransmitMaxCount);
        checkResult = tftpAndCheckCRC(subProgressMonitor, bFile, openTftpCrcCheck);
        if (checkResult)
          break; 
        retransmitMaxCount++;
      } 
    } else {
      checkResult = tftpAndCheckCRC(subProgressMonitor, bFile, openTftpCrcCheck);
    } 
    return checkResult;
  }
  
  protected boolean jtagAndCheckCRC(File partfile) throws IOException, ReceiveTimeOutException {
    boolean openTftpCrcCheck = false;
    if (ToolUtils.isCmdLine()) {
      openTftpCrcCheck = (PreferenceConfig.isTftpCrcCheck() && !FastbootView.isChipBranchSeries7());
    } else {
      openTftpCrcCheck = FastbootPreferences.isOpenTftpCrcCheck();
    } 
    if (openTftpCrcCheck) {
      int retransmitCount = 0;
      while (retransmitCount < 3) {
        boolean result = sendFileByJTAG(partfile);
        if (!result)
          return false; 
        result = checkJtagCrc32(partfile);
        if (result)
          return result; 
        retransmitCount++;
      } 
    } else {
      return sendFileByJTAG(partfile);
    } 
    return true;
  }
  
  protected boolean checkJtagCrc32(File partfile) throws IOException, ReceiveTimeOutException {
    boolean result = false;
    long fileCrcValue = CrcCommand.calculateCRC(partfile);
    if (this.crcLogger != null)
      this.crcLogger.logStart(partfile.getName(), partfile.length()); 
    if (sendCommand((SerialCommand)CrcCommand.sendCRCCommand(partfile, getHexStartAddress()))) {
      String crcReturnStr = getCurReceived();
      result = CrcCommand.compareFileCRC(fileCrcValue, CrcCommand.receiveCRC(crcReturnStr));
      if (!result)
        warningln("Warn:CRC check failed, JTAG download again."); 
    } 
    if (this.crcLogger != null)
      this.crcLogger.logEnd(); 
    return result;
  }
  
  protected boolean sendFileByJTAG(File file) {
    boolean checkFile = (file == null || !file.exists() || file.isDirectory());
    if (checkFile)
      return false; 
    infoln("\nSend command:\tDownloading File by JTAG: " + file.getName());
    boolean result = CallLibJTAGWorker.sendFileByJtag(file.getPath(), Long.parseLong(getHexStartAddress(), 16));
    return result;
  }
  
  protected boolean waitBootRun(int timeMS) {
    long startTime = System.currentTimeMillis();
    int offset = 0;
    String temp = null;
    CharSequence subSequence = null;
    StringBuilder i2csBuilder = new StringBuilder();
    StringBuilder usbsBuilder = new StringBuilder();
    long timeout = 30000L;
    TransferWay transferWay = this.burner.getBurnContext().getTransferWay();
    if (transferWay == TransferWay.USBBootrom || transferWay == TransferWay.USBNonNude) {
      CallLibUSBWorker.closeUsb(transferWay);
      sleep(timeMS);
      boolean openUsbResult = CallLibUSBWorker.openUsb(this.uimonitor, timeout, transferWay, 
          BurnConfig.getUsbSerialNumber());
      if (!openUsbResult)
        return false; 
    } 
    while (System.currentTimeMillis() - startTime < timeout) {
      temp = getTempReceive(temp, i2csBuilder, usbsBuilder, transferWay);
      int indexOf = -1;
      if (temp != null && temp.length() > offset) {
        int len = temp.length();
        subSequence = temp.subSequence(offset, len);
        offset += subSequence.length();
        if (subSequence.toString().indexOf("\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024") == -1) {
          received(subSequence.toString());
        } else {
          receivedln(subSequence.toString());
        } 
        indexOf = temp.toLowerCase().indexOf("start download process.");
        if (UnknownDevice.isOpenLightDebug() && 
          temp.toString().contains("*** Not upport current start mode")) {
          AbstractBurnerWorker.lightCurrentStateMap.put(AbstractBurnerWorker.Light_E.FLASH_AND_IMAGE_MATCH.name(), 
              Integer.valueOf(AbstractBurnerWorker.Light_State_E.abnormal.ordinal()));
          updateLightState();
        } 
        if (indexOf != -1) {
          if (UnknownDevice.isOpenLightDebug())
            readBootInfo(temp); 
          received("\n");
          return true;
        } 
      } 
    } 
    if (UnknownDevice.isOpenLightDebug()) {
      lightCurrentStateMap.put(AbstractBurnerWorker.Light_E.FASTBOOT_START.name(), Integer.valueOf(AbstractBurnerWorker.Light_State_E.abnormal.ordinal()));
      updateLightState();
    } 
    returnError(Messages.ERROR_INFO_Failed_To_Wait_Boot_Running);
    return false;
  }
  
  protected String getTempReceive(String temp, StringBuilder i2csBuilder, StringBuilder usbsBuilder, TransferWay transferWay) {
    String tempReceived = temp;
    if (transferWay == TransferWay.I2CBootrom) {
      try {
        receiveBytesByI2c(i2csBuilder, 256);
        tempReceived = i2csBuilder.toString();
      } catch (IOException iOException) {}
    } else if (transferWay == TransferWay.USBBootrom || transferWay == TransferWay.USBNonNude) {
      try {
        receiveBytesByUsbIgnore(usbsBuilder, 512, transferWay);
        tempReceived = usbsBuilder.toString();
      } catch (IOException iOException) {}
    } else {
      socketReceived();
      tempReceived = serialReceived.toString();
    } 
    return tempReceived;
  }
  
  protected boolean waitBootRunCA() {
    long startTime = System.currentTimeMillis();
    int offset = 0;
    String temp = null;
    CharSequence subSequence = null;
    while (System.currentTimeMillis() - startTime < 10000L) {
      onAdataAvailable(serialReceived);
      temp = serialReceived.toString();
      if (temp.length() > offset) {
        int len = temp.length();
        subSequence = temp.subSequence(offset, len);
        offset += subSequence.length();
        if (temp.toString().indexOf("     ") == -1) {
          received(subSequence.toString());
        } else {
          try {
            sendBytes(new byte[] { -86 });
          } catch (IOException e) {
            return false;
          } 
          receivedln(subSequence.toString());
        } 
      } 
      int indexOf = temp.toLowerCase().indexOf("start download process.");
      if (indexOf != -1) {
        this.ubootStartLog = temp;
        received("\n");
        return true;
      } 
    } 
    returnError(Messages.ERROR_INFO_Failed_To_Wait_Boot_Running);
    return false;
  }
  
  protected boolean waitOpenUsb(IProgressMonitor uiMonitor, TransferWay curTransferWay) {
    CallLibUSBWorker.closeUsb(curTransferWay);
    while (true) {
      this.uimonitor.subTask("Waiting Open Usb");
      boolean openUsbResult = CallLibUSBWorker.openUsb(this.uimonitor, 30000L, curTransferWay, 
          BurnConfig.getUsbSerialNumber());
      if (openUsbResult) {
        infoln("\nOpen Usb successfully!");
        return true;
      } 
      infoln("\nOpen Usb Failed!");
      if (uiMonitor.isCanceled())
        return false; 
      try {
        Thread.sleep(200L);
      } catch (InterruptedException interruptedException) {}
    } 
  }
  
  protected boolean bootromByNonNude(IProgressMonitor uimonitor) {
    if (!downloadFile()) {
      returnError(Messages.ERROR_INFO_Failed_To_Send_Start_Frame);
      return false;
    } 
    if (getFlashType() != DDR)
      infoln("Boot started successfully!"); 
    return true;
  }
  
  protected NetCommand[] buildNetCommands(TransferWay transferWay) {
    IPreferenceStore preferences = FastbootPreferences.getPreferenceStore();
    String ethactID = preferences.getString("vendor.burn.config.ethact");
    NetCommand[] commands = null;
    if (ethactID == null)
      commands = new NetCommand[5]; 
    if (ethactID != null && ethactID.equalsIgnoreCase(NetCommand.getKeyEthact()[0]))
      commands = new NetCommand[5]; 
    if (ethactID != null && !ethactID.equalsIgnoreCase(NetCommand.getKeyEthact()[0]))
      commands = new NetCommand[6]; 
    String serverIp = TFTPPreferences.getPRETFTPSERVERIP();
    if (commands != null) {
      if (serverIp != null && !serverIp.isEmpty()) {
        commands[0] = NetCommand.setServerIP(serverIp);
      } else {
        commands[0] = NetCommand.setServerIP(this.localConfig.getServerIp());
      } 
      if (transferWay == TransferWay.UsbTransNet) {
        commands[1] = NetCommand.setClientMAC("");
      } else {
        commands[1] = NetCommand.setClientMAC(getBoardConfig().getMac());
      } 
      commands[2] = NetCommand.setClientIP(getBoardConfig().getIp());
      commands[3] = NetCommand.setClientNetMask(getBoardConfig().getSubnetMask());
      commands[4] = NetCommand.setClientGateway(getBoardConfig().getGateway());
      if (ethactID != null) {
        if (ethactID.equalsIgnoreCase(NetCommand.getKeyEthact()[0]))
          return commands; 
        if (ethactID.equalsIgnoreCase(NetCommand.getKeyEthact()[1])) {
          commands[5] = NetCommand.setEthact("0");
        } else if (ethactID.equalsIgnoreCase(NetCommand.getKeyEthact()[2])) {
          commands[5] = NetCommand.setEthact("1");
        } else if (ethactID.equalsIgnoreCase(NetCommand.getKeyEthact()[3])) {
          commands[5] = NetCommand.setEthact("2");
        } else if (ethactID.equalsIgnoreCase(NetCommand.getKeyEthact()[4])) {
          commands[5] = NetCommand.setEthact("3");
        } else {
          return commands;
        } 
      } 
    } 
    return commands;
  }
  
  protected boolean setupNetwork(TransferWay transferWay) throws ReceiveTimeOutException {
    if (ToolUtils.getProductLine() == ProductLine.PRODUCTLINE.CAM) {
      infoln("waiting phy ready, it will take 8s.");
      try {
        Thread.sleep(8000L);
      } catch (InterruptedException interruptedException) {}
    } 
    try {
      if (transferWay == TransferWay.UsbTransNet || this.burner.getBurnContext().isUsbToEthernet()) {
        String usbTransNetCmd = "usb start";
        if (!sendCommand(usbTransNetCmd)) {
          returnError(Messages.Fastboot_NotSupportUsbTransNetWay);
          return false;
        } 
      } 
      NetCommand[] commands = buildNetCommands(transferWay);
      if (commands == null)
        return false; 
      for (NetCommand command : commands) {
        if (!sendCommand((SerialCommand)command))
          return false; 
      } 
      if (!setupTftpPort())
        return false; 
    } catch (IOException iOException) {}
    FastbootView.setStartNetwork(false);
    return true;
  }
  
  protected boolean setupTftpPort() throws IOException, ReceiveTimeOutException {
    String portConfigurable = PropertiesUtility.queryForString("portConfigurable");
    int port = CrcCommand.getTFTPPort();
    String changePortCommand = "setenv tftpdstp " + String.valueOf(port);
    if (ToolUtils.isCmdLine()) {
      if (Boolean.valueOf(portConfigurable).booleanValue() && 
        !sendCommand(changePortCommand))
        return false; 
    } else {
      sendCommand(changePortCommand);
    } 
    return true;
  }
  
  public FlashType loadBootMode() throws ReceiveTimeOutException {
    try {
      if (!sendCommand((SerialCommand)INFOCommand.getBootmode()))
        return null; 
    } catch (IOException e) {
      return null;
    } 
    String strCurrentLog = getCurReceived().toLowerCase();
    if (strCurrentLog.indexOf("nand") > -1)
      return FlashType.nand; 
    if (strCurrentLog.indexOf("spi") > -1)
      return FlashType.spi; 
    if (strCurrentLog.indexOf("emmc") > -1)
      return FlashType.emmc; 
    return null;
  }
  
  public long getBlockSize(FlashType type, ChipID chipID, boolean isFastboot) {
    long chipSize = getConstantsBlockSize(type);
    if (chipSize >= 0L)
      return chipSize; 
    try {
      if (sendCommand((SerialCommand)INFOCommand.info(type))) {
        String curReceived = getCurReceived();
        setStrNandInfo(curReceived);
        long blockSize = WorkerUtility.ReceiveHelper.getBlockSize(curReceived);
        if (FlashType.nand == type) {
          setPageSize(WorkerUtility.ReceiveHelper.getPageSize(curReceived));
          setNandBlockSize(blockSize);
          if (isPage() && !isFastboot)
            return getPageSize(); 
          return getNandBlockSize();
        } 
        if (FlashType.spi == type) {
          setSpiBlockSize(blockSize);
          return getSpiBlockSize();
        } 
      } else if (this.burner.getBurnContext().getPlatformType() == PlatformType.CA) {
        setStrNandInfo(this.ubootStartLog);
        long blockSize = WorkerUtility.ReceiveHelper.getBlockSize(this.ubootStartLog);
        if (FlashType.nand == type) {
          setPageSize(WorkerUtility.ReceiveHelper.getPageSize(this.ubootStartLog));
          setNandBlockSize(blockSize);
          if (isPage() && !isFastboot)
            return getPageSize(); 
          return getNandBlockSize();
        } 
        if (FlashType.spi == type) {
          setSpiBlockSize(blockSize);
          return getSpiBlockSize();
        } 
      } else {
        return 0L;
      } 
    } catch (IOException e) {
      return 0L;
    } catch (ReceiveTimeOutException e) {
      return 0L;
    } 
    return 0L;
  }
  
  private long getConstantsBlockSize(FlashType type) {
    if (FastbootView.isChipBranchSeries7()) {
      if (FlashType.nand == type) {
        setNandBlockSize(4096L);
      } else if (FlashType.spi == type) {
        setSpiBlockSize(4096L);
      } 
      return 4096L;
    } 
    if (FlashType.fuzzyCompareFlash(type, FlashType.emmc))
      return 512L; 
    if (FlashType.nand != type && FlashType.spi != type)
      return 0L; 
    return -1L;
  }
  
  public long getWriteLength(FlashType flashType, long blockSize, long fileSize) {
    long writeLength = 0L;
    if (fileSize % blockSize != 0L) {
      writeLength = blockSize * (fileSize / blockSize + 1L);
    } else {
      writeLength = fileSize;
    } 
    if (FlashType.fuzzyCompareFlash(flashType, FlashType.emmc))
      writeLength = (fileSize + blockSize - 1L) / blockSize; 
    return writeLength;
  }
  
  private void sleep(int timeMS) {
    if (timeMS < 1)
      return; 
    try {
      Thread.sleep(timeMS);
    } catch (InterruptedException interruptedException) {}
  }
  
  protected int printBuffer(int offset) {
    onAdataAvailable(serialReceived);
    if (offset < 0)
      offset = 0; 
    int length = serialReceived.length();
    if (length > offset)
      received(serialReceived.substring(offset, length)); 
    return length;
  }
  
  protected boolean downloadFileSerial(File file) throws IOException, ReceiveTimeOutException {
    infoln("[Download File by Serial]\nfile = " + file.getCanonicalPath() + "\naddress = " + getHexStartAddress());
    infoln("Serial Download Command = " + file.getCanonicalPath());
    return downloadFile(file.getCanonicalPath());
  }
  
  protected boolean downloadFileSerial(SubMonitor subProgressMonitor, File file, boolean isSbl) throws IOException, ReceiveTimeOutException {
    try {
      subProgressMonitor.beginTask("", 100);
      subProgressMonitor.subTask("Serial File Transfer");
      infoln("[Download File by Serial]\nfile = " + file.getCanonicalPath() + "\naddress = " + 
          getHexStartAddress());
      infoln("Serial Download Command = " + file.getCanonicalPath());
      infoln("                    length=" + file.length());
      infoln("                    isSBL=" + isSbl);
      return sendByXModem(getHexStartAddressForInt(), file.getCanonicalPath());
    } finally {
      subProgressMonitor.done();
    } 
  }
  
  public void socketReceived() {
    onAdataAvailable(serialReceived);
  }
  
  public boolean sendByXModem(int bootDdrAddr, String fileName) throws IOException {
    if (this.boardConfig.isSocket()) {
      long socketFd = CallLibBootromWorker.getSocketFd();
      if (socketFd != -1L)
        CallLibBootromWorker.destroyDevice(socketFd); 
      boolean isSuccess = CallLibBootromWorker.createSerialPortServer(this.boardConfig.getSerialServerIp(), this.boardConfig
          .getSerialServerPort());
      if (!isSuccess) {
        infoln("Create serial port server failed.");
        return false;
      } 
      socketFd = CallLibBootromWorker.getSocketFd();
      infoln("Create serial port server success." + socketFd);
      if (!sendUserNameAndPassWord(this.boardConfig, false))
        return false; 
      for (int i = 0; i < this.RETY_TIMES; i++) {
        isSuccess = CallLibBootromWorker.downloadData(socketFd, fileName, bootDdrAddr);
        if (isSuccess)
          return true; 
      } 
      return false;
    } 
    return CallLibBootromWorker.downloadData(CallLibBootromWorker.getComFd(), fileName, bootDdrAddr);
  }
  
  public static int crc_16(byte[] data) {
    int crc = 0;
    for (int counter = 0; counter < data.length; counter++) {
      int d = data[counter] & 0xFF;
      crc = crc << 8 ^ crc16_table[(crc >> 8 ^ d) & 0xFF];
    } 
    return crc & 0xFFFF;
  }
  
  protected boolean upload(SubMonitor subProgressMonitor, File file, String hexAddress, String hexLength) throws IOException, ReceiveTimeOutException {
    int length = Integer.parseInt(hexLength, 16);
    int adress = Integer.parseInt(hexAddress, 16);
    infoln("Download :\t" + file.getCanonicalPath());
    deleteContent(serialReceived);
    deleteContent(i2csReceived);
    deleteContent(usbsReceived);
    return updateloadFile(getHexStartAddressForInt(), adress, length, file.getCanonicalPath());
  }
  
  public boolean loadVersionInfo(TransferWay transferWay) {
    if (transferWay == TransferWay.UsbAndSerial || 
      FastbootView.isSupportUsbNonNude(transferWay)) {
      this.version = "";
      return true;
    } 
    try {
      if (!sendCommand((SerialCommand)INFOCommand.getVersion()))
        return false; 
    } catch (IOException iOException) {
    
    } catch (ReceiveTimeOutException receiveTimeOutException) {}
    this.version = WorkerUtility.ReceiveHelper.getVersion(getCurReceived());
    return (this.version != null);
  }
  
  private void loadBlockSizes(FlashType flash, ChipID chipID, boolean isFastboot) {
    if (flash == FlashType.spi)
      getBlockSize(FlashType.spi, chipID, isFastboot); 
    if (flash == FlashType.nand)
      getBlockSize(FlashType.nand, chipID, isFastboot); 
  }
  
  protected boolean burnSinglePartByI2c(IProgressMonitor uimonitor, File file, String startAddress, FlashType flashType, boolean isNakedBurn, ChipID chipID) throws IOException, ReceiveTimeOutException {
    boolean result = true;
    long startAddr = Long.parseLong(startAddress, 16);
    if (!file.exists()) {
      returnError(NLS.bind(Messages.PartitionBurnerWork_I2c_Not_Exist, "File"));
      return false;
    } 
    if (this.burner.getBurnContext().getTransferWay() == TransferWay.I2CBootrom) {
      if (isNakedBurn) {
        if (this.burner.getBurnContext().getPlatformType() == PlatformType.NonCA) {
          if (!bootDownload(SubMonitor.convert(uimonitor, "", 86), file
              .getPath()))
            return false; 
        } else if (!programmerDownload(SubMonitor.convert(uimonitor, "", 86), file.getPath(), ((AbstractBurner)this.burner)
            .getProgrammerFile(), null)) {
          return false;
        } 
        if (flashType == FlashType.ddr) {
          infoln(msg_BootBurnedSuccessfully);
          try {
            Thread.sleep(2000L);
          } catch (InterruptedException e) {
            warningln("Error occurs while waiting DDR run.");
            return false;
          } 
          received(getCurReceived());
          uimonitor.done();
          return true;
        } 
        uimonitor.setTaskName("Loading Info");
        if (loadVersionInfo(this.burner.getBurnContext().getTransferWay())) {
          FlashType loadBootMode = loadBootMode();
          if (flashType != loadBootMode && loadBootMode != null) {
            returnError(NLS.bind(Messages.ERROR_INFO_Invalid_Mode_And_RealMode, loadBootMode.name()));
            return false;
          } 
        } else {
          warningln(msg_FailedToLoadVersion);
          return false;
        } 
        uimonitor.worked(1);
        uimonitor.setTaskName("Burning Boot");
        long blockSize = getBlockSize(flashType, chipID, false);
        if (blockSize == 0L) {
          String error = "Failed to load blocksize for " + flashType.name();
          returnError(error);
          return false;
        } 
        long fileSize = file.length();
        long eraseLength = fileSize;
        if (eraseLength % blockSize != 0L)
          eraseLength = (eraseLength / blockSize + 1L) * blockSize; 
        long writeLength = 0L;
        if (fileSize % blockSize != 0L) {
          writeLength = blockSize * (fileSize / blockSize + 1L);
        } else {
          writeLength = fileSize;
        } 
        if (!bootBurn(SubMonitor.convert(uimonitor, "", 12), flashType, 
            Long.toHexString(eraseLength), Long.toHexString(writeLength), ChipID.chip0)) {
          returnError("Failed to burn boot.");
          return false;
        } 
        infoln(msg_BootBurnedSuccessfully);
        sendCommandNoReturnValue("reset");
        I2cCommand.I2cCloseChannel();
        infoln("\nI2C channels were closed successfully.");
        return true;
      } 
      if (FastbootView.isChipBranchSeries6()) {
        if (this.burner.getBurnContext().getPlatformType() == PlatformType.NonCA) {
          result = bootDownload(null);
        } else {
          result = programmerDownload(null, ((AbstractBurner)this.burner).getProgrammerFile());
        } 
        if (!result) {
          returnError("Failed to Download Boot!");
          return false;
        } 
        if (loadVersionInfo(this.burner.getBurnContext().getTransferWay())) {
          loadBlockSizes(flashType, ChipID.chip0, false);
        } else if (this.burner.getBurnContext().getPlatformType() == PlatformType.CA) {
          loadBlockSizes(flashType, ChipID.chip0, false);
          warningln("Warning: Failed to load version info!");
        } else {
          returnError("Failed to load version info!");
          return false;
        } 
        I2cCommand.I2cCloseChannel();
      } 
    } 
    if (I2cCommand.I2cOpenChannel()) {
      infoln("\nI2C channels open success, the bit rate is " + I2cCommand.getBaudRate() + ", device address is 0x" + 
          Integer.toHexString(I2cCommand.getDeviceAddress()));
    } else {
      return false;
    } 
    int i2cDelay = FastbootPreferences.getI2cDelay();
    File partFile = copyComFileToToolPath(uimonitor, file);
    if (partFile == null)
      return false; 
    File compressFile = GZipUtils.compress(uimonitor, partFile, false, true);
    if (compressFile == null) {
      returnError(NLS.bind(Messages.ERROR_INFO_DISK_SPACE_LOW, partFile));
      return false;
    } 
    long compressFileLen = compressFile.length();
    BufferedRandomAccessFile inFile = null;
    try {
      inFile = new BufferedRandomAccessFile(compressFile, "r");
      result = I2cCommand.sendUpgradeStart((int)compressFileLen, true);
      if (!result) {
        closeI2cAndDeleteTempFile(compressFile, partFile);
        return result;
      } 
      int dataCount = (int)((compressFileLen + 124L - 1L) / 124L);
      int dataSpace = (int)compressFileLen % 124;
      uimonitor.setTaskName("Downloading File");
      uimonitor.beginTask("Burn the " + file.getName(), dataCount);
      uimonitor.setTaskName("Downloading File");
      infoln("\n[Download File by I2c]");
      infoln("file = " + compressFile.getCanonicalPath());
      infoln("address = " + startAddr);
      infoln("fileLength = " + compressFileLen);
      int totalOffset = 0;
      double rateLast = 0.0D;
      int total = (int)compressFileLen;
      byte[] data = new byte[124];
      int upgradeIndex = 0;
      if (dataCount == 0) {
        int readLen = inFile.read(data, 0, data.length);
        if (readLen != data.length)
          return false; 
        result = I2cCommand.sendUpgradeRun(upgradeIndex, data);
        if (!result) {
          closeI2cAndDeleteTempFile(compressFile, partFile);
          return result;
        } 
        upgradeIndex++;
      } else {
        for (int j = 0; j < dataCount; j++) {
          if (uimonitor.isCanceled()) {
            boolean handleCancel = handleCancel();
            if (handleCancel) {
              closeI2cAndDeleteTempFile(compressFile, partFile);
              if (partFile.exists())
                FileUtils.deleteFile(partFile); 
              return false;
            } 
          } 
          int len = 0;
          if (j == dataCount - 1) {
            if (dataSpace == 0) {
              len = 124;
            } else {
              len = dataSpace;
              data = new byte[124];
            } 
          } else {
            len = 124;
          } 
          int read = inFile.read(data, 0, len);
          if (read < 0)
            System.out.println("Read Error"); 
          if (upgradeIndex == 256)
            upgradeIndex = 0; 
          result = I2cCommand.sendUpgradeRun(upgradeIndex, data);
          if (!result) {
            closeI2cAndDeleteTempFile(compressFile, partFile);
            if (partFile.exists())
              FileUtils.deleteFile(partFile); 
            return result;
          } 
          if (upgradeIndex != 0 && upgradeIndex % 10 == 0) {
            int status = I2cCommand.sendGetUpgradeStatus(i2cDelay, monitor);
            if (status == -1) {
              closeI2cAndDeleteTempFile(compressFile, partFile);
              returnError(Messages.PartitionBurnerWork_I2c_Upgrade_Fail);
              return false;
            } 
          } 
          upgradeIndex++;
          totalOffset += len;
          received("#");
          double temp = totalOffset * 100.0D / total;
          if (temp - rateLast >= 1.0D && (int)temp != 100) {
            receivedln(" ---- " + (int)temp + "%");
            rateLast = temp;
          } 
          uimonitor.worked(1);
        } 
      } 
    } catch (Exception exception) {
    
    } finally {
      if (inFile != null)
        inFile.close(); 
    } 
    receivedln(" ---- 100%");
    result = I2cCommand.sendUpgradeFinish((int)startAddr, chipID);
    if (!result) {
      closeI2cAndDeleteTempFile(compressFile, partFile);
      return result;
    } 
    int waitWriteTimeout = FastbootPreferences.getI2cTimeout();
    result = checkUpgradeStatus(file, waitWriteTimeout, i2cDelay);
    if (!result) {
      closeI2cAndDeleteTempFile(compressFile, partFile);
      return result;
    } 
    closeI2cAndDeleteTempFile(compressFile, partFile);
    return true;
  }
  
  protected void closeI2cAndDeleteTempFile(File compressFile, File comFatherFile) {
    if (compressFile != null && compressFile.exists())
      FileUtils.deleteFile(compressFile); 
    if (comFatherFile != null && comFatherFile.exists())
      FileUtils.deleteFile(comFatherFile); 
  }
  
  protected boolean checkUpgradeStatus(File file, long timeout, int i2cDelay) {
    long oldTime = System.currentTimeMillis();
    long currentTime = 0L;
    boolean burnSuccess = false;
    receivedln("Write data to flash...");
    onAdataAvailable(serialReceived);
    while (currentTime - oldTime < timeout) {
      int status = I2cCommand.sendGetUpgradeStatus(i2cDelay, monitor);
      if (status == -1) {
        returnError(Messages.PartitionBurnerWork_I2c_Upgrade_Fail);
        onAdataAvailable(serialReceived);
        if (serialReceived != null && serialReceived.length() > 0)
          receivedln(serialReceived.toString()); 
        return false;
      } 
      if (status != 0 && 
        status == 1) {
        receivedln("The " + file.getName() + " burning successfully.");
        burnSuccess = true;
        break;
      } 
      currentTime = System.currentTimeMillis();
    } 
    if (serialReceived != null && serialReceived.length() > 0)
      receivedln(serialReceived.toString()); 
    if (!burnSuccess)
      returnError(Messages.PartitionBurnerWork_I2c_Upgrade_Write_Flash_Timeout); 
    return burnSuccess;
  }
  
  public boolean initBoard(ArrayList<FlashType> hasFlashTypes) throws IOException, ReceiveTimeOutException {
    debugPrintln("InitBoard Start");
    if (this.burner.getBurnContext().getPlatformType() == PlatformType.NonCA) {
      if (!bootDownload(null))
        return false; 
    } else if (!programmerDownload(null, ((AbstractBurner)this.burner).getProgrammerFile())) {
      return false;
    } 
    this.version = null;
    boolean loadVersionInfo = loadVersionInfo(this.burner.getBurnContext().getTransferWay());
    if (!loadVersionInfo || this.version == null)
      if (this.burner.getBurnContext().getPlatformType() == PlatformType.CA) {
        warningln("Warning: Failed to load version info!");
      } else {
        returnError("Failed to load version info!");
        return false;
      }  
    TransferWay transferWay = this.burner.getBurnContext().getTransferWay();
    if (isNeedSetupNet(transferWay))
      setupNetwork(transferWay); 
    return true;
  }
  
  public boolean isNeedSetupNet(TransferWay transferWay) {
    boolean result = (transferWay == TransferWay.Net || transferWay == TransferWay.UsbTransNet || transferWay == TransferWay.NetNonNude || transferWay == TransferWay.JTAGAndNet);
    return result;
  }
  
  protected void switchTFTPDownloadHome(File file) {
    if (file == null)
      return; 
    if (file.isFile())
      file = file.getParentFile(); 
    if (!TFTPService.TFTPSERVICE.isStart()) {
      InetAddress address = null;
      try {
        address = InetAddress.getByName("x.x.x.x");
      } catch (UnknownHostException unknownHostException) {}
      int port = FastbootPreferences.getInt("vendor.burn.tftp.port");
      try {
        boolean result = TFTPService.TFTPSERVICE.start(file, address, ToolUtils.getPortCmd(), port);
        if (!result)
          warningln(Messages.PartitionBurner_msg_start_Tftp_server_failed_on_port_69); 
        if (file.isDirectory()) {
          TFTPService.TFTPSERVICE.setServerDirectory(file);
          infoln("\tTftp server Download Home switched to <" + file.getCanonicalPath() + ">");
        } 
      } catch (IOException iOException) {}
    } 
  }
  
  protected File copyComFileToToolPath(IProgressMonitor uiMonitor, File file) {
    boolean tftpOccupied = this.burner.getBurnContext().isTftpOccupied();
    String partFilePath = null;
    String toolPath = null;
    if (tftpOccupied) {
      partFilePath = file.getPath();
    } else {
      toolPath = ToolUtils.getInstallPath();
      partFilePath = toolPath + toolPath + File.separator;
    } 
    File partFile = new File(partFilePath);
    boolean copy = FileUtils.copy(uiMonitor, file, partFile);
    if (!copy) {
      returnError(NLS.bind(Messages.ERROR_INFO_DISK_SPACE_LOW, partFile));
      return null;
    } 
    return partFile;
  }
  
  public void printCrcValue(File file) {
    if (file == null)
      return; 
    try {
      long crcValue = CrcCommand.calculateCRC16(file);
      infoln("\nCRC: " + file.getName() + " 0x" + Long.toHexString(crcValue));
    } catch (IOException iOException) {}
  }
  
  protected void getTempFileDivisionSize(TransferWay curTransferWay) throws IOException, ReceiveTimeOutException {
    String ddrFreeSizeStr = getDDRFreeParam();
    long ddrFreeSize = parseLong(getDDRFreeParam("size:", ddrFreeSizeStr, curTransferWay));
    String ddrInitSize = getHexStartAddress();
    if (ddrInitSize.startsWith("0x"))
      ddrInitSize = ddrInitSize.substring(ddrInitSize.indexOf("0x") + "0x".length()); 
    long preferemcesSplitFileSize = ToolUtils.getSplitFileSize();
    if (ddrFreeSize == 0L) {
      tempFileDivisionSize = preferemcesSplitFileSize;
    } else if (ddrFreeSize / 2L > 33554432L) {
      tempFileDivisionSize = (ddrFreeSize - Long.parseLong(ddrInitSize, 16)) / 2L;
    } else if (ddrFreeSize / 2L < 33554432L) {
      tempFileDivisionSize = ddrFreeSize;
    } else {
      tempFileDivisionSize = 33554432L;
    } 
  }
  
  public BoardConfig getBoardConfig() {
    return this.boardConfig;
  }
}
